// Copyright 2019 github.com. All rights reserved.
// Use of this source code is governed by github.com.

package v1

import (
	"context"
	"fmt"
	"git.getensh.com/common/gopkgs/cache"
	"google.golang.org/grpc/status"
	"net/http"
	"property-system-gateway/errors"
	param_v1 "property-system-gateway/param/v1"
	"property-system-gateway/pb"
	"property-system-gateway/pb/v1"
	"property-system-gateway/utils"
	"strings"
	"time"

	"git.getensh.com/common/gopkgs/logger"
	"git.getensh.com/common/gopkgs/tasker/httptasker"
	"git.getensh.com/common/gopkgs/util"
	"github.com/dgrijalva/jwt-go"

	"property-system-gateway/parser"

	"git.getensh.com/common/gopkgs/jwtwrapper"
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
)

func phonePasswordLogin(req *param_v1.LoginRequest) (param_v1.LoginResponse, error) {
	resp := param_v1.LoginResponse{}
	rpcReq := &v1.LoginByPhoneRequest{
		Phone:    req.User,
		Password: req.Password,
	}

	rpcRsp, err := pb.System.LoginByPhone(context.Background(), rpcReq)
	if err != nil {
		s, _ := json.MarshalToString(req)
		logger.Error("func",
			zap.String("call", "pb.System.LoginByPhone"),
			zap.String("params", s),
			zap.String("error", err.Error()))
		return resp, err
	}
	if len(rpcRsp.List) == 0 {
		return resp, errors.ErrRecordNotFound
	}
	// 只有一条记录
	if len(rpcRsp.List) == 1 {
		now := time.Now()
		routers := map[string]bool{}
		for _, v := range rpcRsp.List[0].Permissions {
			routers[v.Router] = true
		}
		subject := map[string]interface{}{
			"user_name":         req.User,
			"cid":               rpcRsp.List[0].Cid,
			"garden_id":         rpcRsp.List[0].GardenId,
			"is_super_group":    rpcRsp.List[0].IsSuperGroup,
			"routers":           routers,
			"garden_name":       rpcRsp.List[0].GardenName,
			"g_permission_time": rpcRsp.List[0].GlobalPermissionTime,
			"u_permission_time": rpcRsp.List[0].UserPermissionTime,
			"single_sign_time":  fmt.Sprintf("%d", now.Unix()),
		}

		str, _ := json.MarshalToString(subject)

		// 生成token
		token, err := jwtwrapper.GenToken(fmt.Sprintf("%d", rpcRsp.List[0].Uid), parser.Conf.Jwt.Issuer, str,
			time.Duration(parser.Conf.Jwt.Seconds)*time.Second)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", fmt.Sprintf("%d", rpcRsp.List[0].Uid)),
				zap.String("error", err.Error()))
			return resp, errors.SystemError
		}
		if err = utils.SetSingleSignTime(utils.GetSingleFlag(false, rpcRsp.List[0].Cid, rpcRsp.List[0].GardenId, rpcRsp.List[0].Uid), now.Unix()); err != nil {
			return resp, err
		}

		permissions := []v1.SystemGroupPermissionData{}
		for _, v := range rpcRsp.List[0].Permissions {
			// 前端控制的权限
			//if v.Front {
			permissions = append(permissions, *v)
			//}
		}
		resp.Data.Uid = rpcRsp.List[0].Uid
		resp.Data.Token = token
		resp.Data.User = req.User
		resp.Data.Permissions = permissions
		resp.Data.IsSuper = rpcRsp.List[0].IsSuperGroup
		resp.Data.OpenimId = rpcRsp.List[0].OpenimId
		return resp, nil
	}

	// 多条记录需要选择小区

	loginResp, err := makeLoginByPhoneResponse(rpcRsp, req.User)
	if err != nil {
		return resp, nil
	}
	str, _ := json.MarshalToString(rpcRsp)
	key := utils.GetKey(utils.SYSTEMPHONELOGINKEY, req.User)
	_, err = cache.Redis().SetEx(key, 600, str)
	if err != nil {
		return resp, errors.RedisError
	}
	resp.PhoneData = loginResp.Data
	return resp, nil
}

// 登录
// @Summary 登录
// @Description 登录
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param body body v1.LoginBody true "登录信息"
// @Success 200 {object} v1.LoginResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/login [post]
func (c *Controller) Login(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.LoginRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.LoginBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		if req.Password == "" {
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.LoginResponse{}
		// 先尝试手机号登录
		if utils.VerifyMobileFormat(req.User) {
			if resp, err := phonePasswordLogin(req); err == nil {
				ctx.JSON(http.StatusOK, resp)
				return nil
			}
		}

		rpcReq := &v1.LoginRequest{
			Username: req.User,
			Password: req.Password,
		}

		rpcRsp, err := pb.System.Login(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.Login"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		routers := map[string]bool{}
		for _, v := range rpcRsp.Permissions {
			routers[v.Router] = true
		}
		now := time.Now()
		subject := map[string]interface{}{
			"user_name":         req.User,
			"cid":               rpcRsp.Cid,
			"garden_id":         rpcRsp.GardenId,
			"is_super_group":    rpcRsp.IsSuperGroup,
			"routers":           routers,
			"garden_name":       rpcRsp.GardenName,
			"g_permission_time": rpcRsp.GlobalPermissionTime,
			"u_permission_time": rpcRsp.UserPermissionTime,
			"single_sign_time":  fmt.Sprintf("%d", now.Unix()),
		}

		str, _ := json.MarshalToString(subject)

		// 生成token
		token, err := jwtwrapper.GenToken(fmt.Sprintf("%d", rpcRsp.Uid), parser.Conf.Jwt.Issuer, str,
			time.Duration(parser.Conf.Jwt.Seconds)*time.Second)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", fmt.Sprintf("%d", rpcRsp.Uid)),
				zap.String("error", err.Error()))
			return errors.SystemError
		}
		if err = utils.SetSingleSignTime(utils.GetSingleFlag(false, rpcRsp.Cid, rpcRsp.GardenId, rpcRsp.Uid), now.Unix()); err != nil {
			return err
		}

		permissions := []v1.SystemGroupPermissionData{}
		for _, v := range rpcRsp.Permissions {
			// 前端控制的权限
			//if v.Front {
			permissions = append(permissions, *v)
			//}
		}
		resp.Data.Uid = rpcRsp.Uid
		resp.Data.Token = token
		resp.Data.User = req.User
		resp.Data.Permissions = permissions
		resp.Data.GardenName = rpcRsp.GardenName
		//fmt.Printf("permissions:%v\n", permissions)
		resp.Data.IsSuper = rpcRsp.IsSuperGroup
		resp.Data.OpenimId = rpcRsp.OpenimId
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 获取openim token
// @Description 获取openim_token
// @Tags openim
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param openim_id query string true "openim_id"
// @Success 200 {object} v1.OpenImTokenResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/openim_token [get]
func (c *Controller) OpenImToken(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.OpenImTokenRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.OpenImTokenQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.OpenImTokenResponse{}
		rpcReq := &v1.OpenImTokenRequest{
			Platform: 5,
			OpenimId: req.OpenimId,
		}

		rpcRsp, err := pb.Thirdparty.OpenImToken(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Thirdparty.OpenImToken"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		resp.Data = *rpcRsp

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 获取openim 用户信息
// @Description 获取openim_用户信息
// @Tags openim
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param openim_id query string true "openim_id"
// @Success 200 {object} v1.OpenImUserInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/openim_user_info [get]
func (c *Controller) OpenImUserInfo(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.OpenImUserInfoRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.OpenImUserInfoQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.OpenImUserInfoResponse{}
		rpcReq := &v1.OpenImUserInfoRequest{
			PlatformId: 5,
			OpenimIds:  []string{req.OpenimId},
		}

		rpcRsp, err := pb.Thirdparty.OpenImUserInfo(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Thirdparty.OpenImUserInfo"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if len(rpcRsp.List) == 0 {
			return errors.ErrRecordNotFound
		}
		resp.Data.NickName = rpcRsp.List[0].NickName
		resp.Data.FaceUrl = rpcRsp.List[0].FaceUrl
		resp.Data.Gender = rpcRsp.List[0].Gender
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 批量获取openim 用户信息
// @Description 批量获取openim_用户信息
// @Tags openim
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param openim_ids query string true "用户openim_id,以逗号分割"
// @Success 200 {object} v1.OpenImUsersInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/openim_users_info [get]
func (c *Controller) OpenImUsersInfo(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.OpenImUsersInfoRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.OpenImUsersInfoQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.OpenImUsersInfoResponse{}
		array := strings.Split(req.OpenimIds, ",")
		rpcReq := &v1.OpenImUserInfoRequest{
			PlatformId: 5,
			OpenimIds:  array,
		}

		rpcRsp, err := pb.Thirdparty.OpenImUserInfo(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Thirdparty.OpenImUserInfo"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if len(rpcRsp.List) == 0 {
			rpcRsp.List = make([]*v1.OpenImUserData, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

func OssObjAdd(inList []string, outList []string) error {
	mreq := v1.OssObjAddRequest{InList: inList, OutList: outList}
	_, err := pb.Common.OssObjAdd(context.Background(), &mreq)
	return err
}

func getOpenimPic(openimId string) string {
	rpcReq := &v1.OpenImUserInfoRequest{
		PlatformId: 5,
		OpenimIds:  []string{openimId},
	}

	rpcRsp, err := pb.Thirdparty.OpenImUserInfo(context.Background(), rpcReq)

	if err != nil {
		logger.Error("func",
			zap.String("call", "pb.Thirdparty.OpenImUserInfo"),
			zap.String("error", err.Error()))

		return ""
	}
	if len(rpcRsp.List) == 0 {
		return ""
	}
	return rpcRsp.List[0].FaceUrl
}

//
// @Summary 设置openim 用户头像
// @Description 设置openim_用户头像
// @Tags openim
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param body body v1.OpenImUserSetFaceBody true " "
// @Success 200 {object} v1.OpenImUserSetFaceResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/openim_user_face [put]
func (c *Controller) OpenImUserSetFace(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.OpenImUserSetFaceRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.OpenImUserSetFaceBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.OpenImUserSetFaceResponse{}

		oldPic := getOpenimPic(req.OpenimId)
		if req.FaceUrl == oldPic {
			ctx.JSON(http.StatusOK, resp)
			return nil
		}
		// 先添加到数据库
		err := OssObjAdd([]string{req.FaceUrl}, []string{})
		if err != nil {
			logger.Error("func",
				zap.String("call", "OssObjAdd"),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		rpcReq := &v1.OpenImUserSetFaceRequest{
			PlatformId: 5,
			OpenimId:   req.OpenimId,
			FaceUrl:    req.FaceUrl,
		}

		_, err = pb.Thirdparty.OpenImUserSetFace(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Thirdparty.OpenImUserSetFace"),
				zap.String("params", s),
				zap.String("error", err.Error()))
			// 出错，删除图片
			OssObjAdd([]string{}, []string{req.FaceUrl})
			return errors.ErrorTransForm(err)
		}
		// 删除旧图片
		_ = OssObjAdd([]string{}, []string{oldPic})

		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// 权限变更后重新获取权限和token
// @Summary 权限变更后重新获取权限和token
// @Description 权限变更后重新获取权限
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Success 200 {object} v1.LoginResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/permission_reget [get]
func (c *Controller) PermissionReget(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.PermissionRegetRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetSubjectValue(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.PermissionRegetResponse{}

		rpcReq := &v1.PermissionRegetRequest{
			Uid:      tokenInfo.Uid,
			Company:  tokenInfo.ByCompany,
			GardenId: tokenInfo.GardenId,
		}

		rpcRsp, err := pb.System.PermissionReget(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.PermissionReget"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		routers := map[string]bool{}
		for _, v := range rpcRsp.Permissions {
			routers[v.Router] = true
		}
		now := time.Now()
		subject := map[string]interface{}{
			"user_name":         tokenInfo.UserName,
			"cid":               tokenInfo.Cid,
			"garden_id":         tokenInfo.GardenId,
			"is_super_group":    rpcRsp.IsSuperGroup,
			"routers":           routers,
			"garden_name":       tokenInfo.GardenName,
			"g_permission_time": rpcRsp.GlobalPermissionTime,
			"u_permission_time": rpcRsp.UserPermissionTime,
			"single_sign_time":  fmt.Sprintf("%d", now.Unix()),
			"by_company":        tokenInfo.ByCompany,
		}

		str, _ := json.MarshalToString(subject)

		// 生成token
		token, err := jwtwrapper.GenToken(fmt.Sprintf("%d", rpcRsp.Uid), parser.Conf.Jwt.Issuer, str,
			time.Duration(parser.Conf.Jwt.Seconds)*time.Second)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", fmt.Sprintf("%d", rpcRsp.Uid)),
				zap.String("error", err.Error()))
			return errors.SystemError
		}
		if err = utils.SetSingleSignTime(utils.GetSingleFlag(tokenInfo.ByCompany, tokenInfo.Cid, tokenInfo.GardenId, rpcRsp.Uid), now.Unix()); err != nil {
			return err
		}

		permissions := []v1.SystemGroupPermissionData{}
		for _, v := range rpcRsp.Permissions {
			// 前端控制的权限
			//if v.Front {
			permissions = append(permissions, *v)
			//}
		}
		resp.Data.Uid = rpcRsp.Uid
		resp.Data.Token = token
		resp.Data.User = tokenInfo.UserName
		resp.Data.Permissions = permissions
		resp.Data.IsSuper = rpcRsp.IsSuperGroup
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

// token
// @Summary 刷新token
// @Description 刷新token
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Success 200 {object} v1.TokenResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/token_refresh [put]
func (c *Controller) TokenRefresh(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.TokenRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenObj, err := jwtwrapper.ParseToken(req.Token)
		if tokenObj == nil {
			return errors.TokenFailedError
		}
		if err != nil {
			switch err.(*jwt.ValidationError).Errors {
			case jwt.ValidationErrorExpired:
				if tokenObj == nil {
					return errors.TokenFailedError
				}
				if time.Now().Unix()-tokenObj.ExpiresAt > 3600 {
					return errors.TokenFailedError
				}

			default:
				return errors.TokenFailedError
			}
		}

		uid := tokenObj.Id
		subject := tokenObj.Subject
		//remberPass := gjson.GetBytes([]byte(subject), "rember_password").Bool()

		// 生成token
		token, err := jwtwrapper.GenToken(uid, parser.Conf.Jwt.Issuer, subject,
			time.Duration(parser.Conf.Jwt.Seconds)*time.Second)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", fmt.Sprintf("%s", uid)),
				zap.String("error", err.Error()))
			return errors.SystemError
		}
		/*
			refreshTokenTime := time.Duration(24*60*60) * time.Second
			if remberPass {
				refreshTokenTime = time.Duration(7*24*60*60) * time.Second
			}
			refreshToken, err := jwtwrapper.GenToken(uid, parser.Conf.Jwt.Issuer, subject,
				refreshTokenTime)*/
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", fmt.Sprintf("%s", uid)),
				zap.String("error", err.Error()))
			return errors.SystemError
		}
		resp := param_v1.TokenResponse{}
		resp.Data = token
		resp.RefreshToken = token
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 获取短信验证码
// @Description 获取短信验证码
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param phone query string true " "
// @Success 200 {object} v1.GetVcodeResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/vcode [get]
func (c *Controller) GetVcode(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.GetVcodeRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, &req.GetVcodeQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	checkPhoneTask := func() error {
		mreq := &v1.CheckPhoneRequest{Phone: req.Phone}
		_, err := pb.System.CheckPhone(ctx, mreq)
		if err != nil {
			return err
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.GetVcodeResponse{}
		rpcReq := &v1.GetVcodeRequest{
			PhoneNumber: req.Phone,
		}

		if true {
			ctx.JSON(http.StatusOK, resp)
			return nil
		}

		rpcRsp, err := pb.Thirdparty.GetVcode(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "Thirdparty.GetVcode"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		resp.Data = *rpcRsp
		resp.Data.Vcode = 0
		ctx.JSON(http.StatusOK, resp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, checkPhoneTask, handleServiceTask)
}

func makeLoginByPhoneToken(rpcRsp *v1.LoginByPhoneReply, phone string) (token string, err error) {
	uid := int64(0)
	var subject map[string]interface{}
	now := time.Now()
	if len(rpcRsp.List) > 1 {
		subject = map[string]interface{}{
			"phone":                phone,
			"should_choose_garden": true,
		}
		uid = 0
	} else {

		routers := map[string]bool{}
		for _, v := range rpcRsp.List[0].Permissions {
			routers[v.Router] = true
		}
		subject = map[string]interface{}{
			"user_name":         rpcRsp.List[0].UserName,
			"cid":               rpcRsp.List[0].Cid,
			"garden_id":         rpcRsp.List[0].GardenId,
			"is_super_group":    rpcRsp.List[0].IsSuperGroup,
			"routers":           routers,
			"phone":             phone,
			"garden_name":       rpcRsp.List[0].GardenName,
			"g_permission_time": rpcRsp.List[0].GlobalPermissionTime,
			"u_permission_time": rpcRsp.List[0].UserPermissionTime,
			"single_sign_time":  fmt.Sprintf("%d", now.Unix()),
		}
		uid = rpcRsp.List[0].Uid
	}

	str, _ := json.MarshalToString(subject)

	token, err = jwtwrapper.GenToken(fmt.Sprintf("%d", uid), parser.Conf.Jwt.Issuer, str,
		time.Duration(parser.Conf.Jwt.Seconds)*time.Second)
	if err != nil {
		logger.Error("func",
			zap.String("call", "util.GenJwtToken"),
			zap.String("args", fmt.Sprintf("%d", rpcRsp.List[0].Uid)),
			zap.String("error", err.Error()))
		return token, errors.SystemError
	}
	if len(rpcRsp.List) == 1 {
		if err = utils.SetSingleSignTime(utils.GetSingleFlag(false, rpcRsp.List[0].Cid, rpcRsp.List[0].GardenId, rpcRsp.List[0].Uid), now.Unix()); err != nil {
			return token, err
		}
	}

	return token, nil
}
func makeLoginByPhoneResponse(rpcRsp *v1.LoginByPhoneReply, phone string) (resp param_v1.LoginByPhoneResponse, err error) {

	token, err := makeLoginByPhoneToken(rpcRsp, phone)
	if err != nil {
		return resp, err
	}
	for i, _ := range rpcRsp.List {
		permissions := []v1.SystemGroupPermissionData{}
		for _, v := range rpcRsp.List[i].Permissions {
			permissions = append(permissions, *v)

		}
		item := param_v1.LoginByPhoneItem{
			Uid:         rpcRsp.List[i].Uid,
			GardenName:  rpcRsp.List[i].GardenName,
			GardenId:    rpcRsp.List[i].GardenId,
			User:        rpcRsp.List[i].UserName,
			Permissions: permissions,
			IsSuper:     rpcRsp.List[i].IsSuperGroup,
			OpenimId:    rpcRsp.List[i].OpenimId,
		}
		resp.Data.List = append(resp.Data.List, item)
		resp.Data.Token = token
	}
	return resp, nil
}

//
// @Summary 手机号登录
// @Description 手机号登录
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param body body v1.LoginByPhoneBody true " "
// @Success 200 {object} v1.LoginByPhoneResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/login_by_phone [post]
func (c *Controller) LoginByPhone(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.LoginByPhoneRequest{}
	loginResp := param_v1.LoginByPhoneResponse{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.LoginByPhoneBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		rpcReq := &v1.LoginByPhoneRequest{
			Phone: req.Phone,
		}

		rpcRsp, err := pb.System.LoginByPhone(ctx, rpcReq)
		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "Thirdparty.GetVcode"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if len(rpcRsp.List) == 0 {
			return errors.ErrRecordNotFound
		}
		loginResp, err = makeLoginByPhoneResponse(rpcRsp, req.Phone)
		if err != nil {
			return err
		}
		str, _ := json.MarshalToString(rpcRsp)
		key := utils.GetKey(utils.SYSTEMPHONELOGINKEY, req.Phone)
		_, err = cache.Redis().SetEx(key, 600, str)
		if err != nil {
			return errors.RedisError
		}

		return nil
	}
	checkVcodeTask := func() error {
		if true {
			ctx.JSON(http.StatusOK, loginResp)
			return nil
		}
		rpcReq := &v1.CheckVcodeRequest{
			PhoneNumber: req.Phone,
			Vcode:       req.Vcode,
		}

		_, err := pb.Thirdparty.CheckVcode(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Thirdparty.CheckVcode"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		ctx.JSON(http.StatusOK, loginResp)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask, checkVcodeTask)
}

//
// @Summary 手机号登录选择账户
// @Description 手机号登录选择账户
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Param id query int true "账户id"
// @Success 200 {object} v1.ChooseUserResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/choose_user [get]
func (c *Controller) ChooseUser(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ChooseUserRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, &req.ChooseUserQuery, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetSubjectValue(ctx)
		if err != nil {
			return err
		}
		resp := param_v1.ChooseUserResponse{}

		// 获取缓存的用户信息
		key := utils.GetKey(utils.SYSTEMPHONELOGINKEY, tokenInfo.Phone)
		str, err := cache.Redis().Get(key)
		if str == "" {
			return errors.SystemError
		}

		loginByPhoneResp := v1.LoginByPhoneReply{}
		json.Unmarshal([]byte(str), &loginByPhoneResp)
		var userInfo *v1.LoginReply

		// 搜索目标小区
		for _, v := range loginByPhoneResp.List {
			if v.Uid == req.Id {
				userInfo = v
				break
			}
		}
		if userInfo == nil {
			return errors.SystemError
		}

		now := time.Now()
		// 构造响应数据
		routers := map[string]bool{}
		for _, v := range userInfo.Permissions {
			routers[v.Router] = true
		}
		subject := map[string]interface{}{
			"user_name":         userInfo.UserName,
			"cid":               userInfo.Cid,
			"garden_id":         userInfo.GardenId,
			"is_super_group":    userInfo.IsSuperGroup,
			"routers":           routers,
			"garden_name":       userInfo.GardenName,
			"g_permission_time": userInfo.GlobalPermissionTime,
			"u_permission_time": userInfo.UserPermissionTime,
			"single_sign_time":  fmt.Sprintf("%d", now.Unix()),
		}

		str, _ = json.MarshalToString(subject)

		// 生成token
		token, err := jwtwrapper.GenToken(fmt.Sprintf("%d", userInfo.Uid), parser.Conf.Jwt.Issuer, str,
			time.Duration(parser.Conf.Jwt.Seconds)*time.Second)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", fmt.Sprintf("%d", userInfo.Uid)),
				zap.String("error", err.Error()))
			return errors.SystemError
		}

		permissions := []v1.SystemGroupPermissionData{}
		for _, v := range userInfo.Permissions {
			permissions = append(permissions, *v)

		}
		if err = utils.SetSingleSignTime(utils.GetSingleFlag(false, userInfo.Cid, userInfo.GardenId, userInfo.Uid), now.Unix()); err != nil {
			return err
		}
		resp.Data.Uid = userInfo.Uid
		resp.Data.Token = token
		resp.Data.User = userInfo.UserName
		resp.Data.IsSuper = userInfo.IsSuperGroup
		resp.Data.Permissions = permissions
		resp.Data.GardenName = userInfo.GardenName
		cache.Redis().Del(key)
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 忘记密码重置密码
// @Description 忘记密码重置密码
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param body body v1.ResetPasswordBody true "信息"
// @Success 200 {object} v1.ResetPasswordResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/reset_password [put]
func (c *Controller) ResetPassword(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ResetPasswordRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.ResetPasswordBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		// 响应数据
		resp := param_v1.ResetPasswordResponse{}
		rpcReq := &v1.ResetPasswordRequest{
			Uid:      req.Uid,
			Phone:    req.Phone,
			Vcode:    req.Vcode,
			Password: req.Password,
		}

		rpcRsp, err := pb.System.ResetPassword(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.ResetPassword"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if rpcRsp.List == nil {
			rpcRsp.List = make([]*v1.ResetPasswordData, 0)
		}
		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		if len(rpcRsp.List) == 1 {
			logReq := OperationLogRequest{
				Module:   ModuleSelf,
				Action:   ActionSelfResetPassword,
				Origin:   nil,
				Target:   nil,
				UserName: "",
				Uid:      rpcRsp.List[0].Uid,
				Cid:      0,
				GardenId: 0,
			}
			go OperationLogAdd(&logReq)
		}

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 修改密码
// @Description 修改密码
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param token header string true " "
// @Param body body v1.ChangePasswordBody true "信息"
// @Success 200 {object} v1.ChangePasswordResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/change_password [put]
func (c *Controller) ChangePassword(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.ChangePasswordRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, nil, nil, nil, &req.ChangePasswordBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetSubjectValue(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.ChangePasswordResponse{}
		rpcReq := &v1.ChangePasswordRequest{
			Uid:         tokenInfo.Uid,
			OldPassword: req.OldPassword,
			NewPassword: req.NewPassword,
		}

		_, err = pb.System.ChangePassword(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.ChangePassword"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)

		logReq := OperationLogRequest{
			Module:   ModuleSelf,
			Action:   ActionSelfChangePassword,
			Origin:   nil,
			Target:   nil,
			UserName: tokenInfo.UserName,
			Uid:      tokenInfo.Uid,
			Cid:      tokenInfo.Cid,
			GardenId: tokenInfo.GardenId,
		}
		go OperationLogAdd(&logReq)
		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 公司账户进入小区
// @Description 公司账户进入小区
// @Tags 用户
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Success 200 {object} v1.CompanyEnterGardenResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/user/company_login [post]
func (c *Controller) CompanyEnterGarden(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.CompanyEnterGardenRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetSubjectValue(ctx)
		if err != nil {
			return err
		}
		if !tokenInfo.IsCompanyEnter ||
			tokenInfo.Cid == 0 ||
			tokenInfo.GardenId == 0 ||
			tokenInfo.Uid == 0 {
			return errors.TokenFailedError
		}
		resp := param_v1.CompanyEnterGardenResponse{}
		rpcReq := &v1.CompanyEnterGardenRequest{
			Cid:      tokenInfo.Cid,
			GardenId: tokenInfo.GardenId,
		}

		rpcRsp, err := pb.System.CompanyEnterGarden(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.CompanyEnterGarden"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		routers := map[string]bool{}
		for _, v := range rpcRsp.Permissions {
			routers[v.Router] = true
		}
		now := time.Now()
		subject := map[string]interface{}{
			"user_name":         tokenInfo.UserName,
			"cid":               tokenInfo.Cid,
			"garden_id":         tokenInfo.GardenId,
			"is_super_group":    true,
			"routers":           routers,
			"garden_name":       rpcRsp.GardenName,
			"by_company":        true,
			"g_permission_time": rpcRsp.GlobalPermissionTime,
			"single_sign_time":  fmt.Sprintf("%d", now.Unix()),
		}

		str, _ := json.MarshalToString(subject)

		// 生成token
		token, err := jwtwrapper.GenToken(fmt.Sprintf("%d", tokenInfo.Uid), parser.Conf.Jwt.Issuer, str,
			time.Duration(parser.Conf.Jwt.Seconds)*time.Second)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.GenJwtToken"),
				zap.String("args", fmt.Sprintf("%d", tokenInfo.Uid)),
				zap.String("error", err.Error()))
			return errors.SystemError
		}
		if err = utils.SetSingleSignTime(utils.GetSingleFlag(true, tokenInfo.Cid, tokenInfo.GardenId, 0), now.Unix()); err != nil {
			return err
		}

		permissions := []v1.SystemGroupPermissionData{}
		for _, v := range rpcRsp.Permissions {
			permissions = append(permissions, *v)
		}
		resp.Data.Uid = tokenInfo.Uid
		resp.Data.Token = token
		resp.Data.User = tokenInfo.UserName
		resp.Data.Permissions = permissions
		resp.Data.IsSuper = true
		resp.Data.Company = true
		resp.Data.GardenName = rpcRsp.GardenName
		//fmt.Printf("permissions:%v\n", permissions)
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 统计
// @Description 统计
// @Tags 首页
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Success 200 {object} v1.StatisticResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/statistic [get]
func (c *Controller) Statistic(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.StatisticRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetSubjectValue(ctx)
		if err != nil {
			return err
		}
		// 响应数据
		resp := param_v1.StatisticResponse{}
		rpcReq := &v1.StatisticRequest{
			GardenId: tokenInfo.GardenId,
		}

		rpcRsp, err := pb.Garden.Statistic(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.Garden.Statistic"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		resp.Data = *rpcRsp
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 个人信息
// @Description 个人信息
// @Tags 个人设置
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Success 200 {object} v1.PersonInfoResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/person [get]
func (c *Controller) PersonInfo(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.PersonInfoRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, nil)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetSubjectValue(ctx)
		if err != nil {
			return err
		}
		if tokenInfo.Uid == 0 {
			return status.Error(10003, "非小区账号")
		}
		// 响应数据
		resp := param_v1.PersonInfoResponse{}
		rpcReq := &v1.UserListRequest{
			GardenId: tokenInfo.GardenId,
			Cid:      tokenInfo.Cid,
			Ids:      []int64{tokenInfo.Uid},
		}

		rpcRsp, err := pb.System.UserList(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.UserList"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}
		if len(rpcRsp.List) == 0 {
			return errors.ErrRecordNotFound
		}

		resp.Data.User = rpcRsp.List[0].UserName
		resp.Data.Phone = rpcRsp.List[0].Phone
		resp.Data.Email = rpcRsp.List[0].Email
		resp.Data.RealName = rpcRsp.List[0].RealName
		resp.Data.Gender = rpcRsp.List[0].Gender
		resp.Data.Department = rpcRsp.List[0].DepartmentName
		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}

//
// @Summary 个人信息修改
// @Description 个人信息修改
// @Tags 个人设置
// @Accept  json
// @Produce  json
// @Param token header string true "token"
// @Success 200 {object} v1.PersonInfoUpdateResponse
// @Failure 500 {object} base.HTTPError
// @Router /api/v1/person [put]
func (c *Controller) PersonInfoUpdate(ctx *gin.Context) {
	// 解析参数
	req := &param_v1.PersonInfoUpdateRequest{}
	parseParamTask := func() error {
		err := util.ShouldBind(ctx, &req.Header, nil, nil, &req.PersonInfoUpdateBody)
		if err != nil {
			logger.Error("func",
				zap.String("call", "util.ShouldBind"),
				zap.String("error", err.Error()))
			return errors.ParamsError
		}
		return nil
	}

	// 业务处理
	handleServiceTask := func() error {
		tokenInfo, err := utils.GetSubjectValue(ctx)
		if err != nil {
			return err
		}
		if tokenInfo.Uid == 0 {
			return status.Error(10003, "非小区账号")
		}
		// 响应数据
		resp := param_v1.PersonInfoUpdateResponse{}
		rpcReq := &v1.UserUpdateRequest{
			GardenId: tokenInfo.GardenId,
			Cid:      tokenInfo.Cid,
			Id:       tokenInfo.Uid,
			Person:   true,
			Phone:    req.Phone,
			Email:    req.Email,
			UserName: req.User,
			RealName: req.RealName,
			Gender:   req.Gender,
		}

		_, err = pb.System.UserUpdate(ctx, rpcReq)

		if err != nil {
			s, _ := json.MarshalToString(req)
			logger.Error("func",
				zap.String("call", "pb.System.UserUpdate"),
				zap.String("params", s),
				zap.String("error", err.Error()))

			return errors.ErrorTransForm(err)
		}

		ctx.JSON(http.StatusOK, resp)

		return nil
	}

	// 执行任务
	httptasker.Exec(ctx, parseParamTask, handleServiceTask)
}
